# -*- coding: utf-8 -*-
# pylint: disable=bare-except
# pylint: disable=too-many-branches

"""
Copyright (c) 2019-2024 Tomasz Łuczak, TeaM-TL

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included
in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

module contains function for reading entries from config file.
Every function read part of config file
- ini_read - general setting for FotoKilof
- resize
- text
- rotate
- crop
- border
- vignette
- color
- normalize
- contrast
- logo
- compose
"""

import configparser

import entries
import common


def main(file_ini, preview_size_list):
    """General settings"""

    # output dictionary
    dict_return = {}

    config = configparser.ConfigParser()
    config.read(file_ini, encoding="utf8")

    # read values from a section
    try:
        file_in = config.get("Main", "path")
    except:
        file_in = ""
    dict_return["file_in_path"] = file_in

    try:
        dir_work = config.get("Main", "work_dir")
    except:
        dir_work = "FotoKilof"
    dict_return["work_dir"] = dir_work

    try:
        file_dir_select = config.getint("Main", "file_dir")
    except:
        file_dir_select = "0"
    dict_return["file_dir_selector"] = entries.parse_list(file_dir_select, (0, 1), 0)

    try:
        exif = config.getint("Main", "exif")
    except:
        exif = "0"
    dict_return["img_exif_on"] = entries.parse_list(exif, (0, 1), 0)

    try:
        custom = config.getint("Main", "custom")
    except:
        custom = 0
    dict_return["img_custom_on"] = entries.parse_list(custom, (0, 1), 0)

    try:
        preview_orig = config.getint("Main", "preview_orig")
    except:
        preview_orig = 400
    dict_return["preview_orig"] = entries.parse_list(
        preview_orig, preview_size_list, 400
    )

    try:
        preview_new = config.getint("Main", "preview_new")
    except:
        preview_new = 400
    dict_return["preview_new"] = entries.parse_list(preview_new, preview_size_list, 400)

    try:
        log_level = config.get("Main", "log_level")
    except:
        log_level = "E"
    dict_return["log_level"] = entries.parse_list(log_level, ("E", "W", "I", "D"), "W")

    try:
        check_version = config.getint("Main", "check_version")
    except:
        check_version = 1
    dict_return["check_version"] = entries.parse_list(check_version, (0, 1), 0)

    return dict_return


def resize(file_ini):
    """Resize/scalling"""

    # output dictionary
    dict_return = {}

    config = configparser.ConfigParser()
    config.read(file_ini, encoding="utf8")

    try:
        resize_on = config.getint("Resize", "on")
    except:
        resize_on = "0"
    dict_return["img_resize_on"] = entries.parse_list(resize_on, (0, 1), 0)

    try:
        resize_variant = config.getint("Resize", "resize")
    except:
        resize_variant = "3"  # FullHD
    dict_return["img_resize"] = entries.parse_list(resize_variant, (1, 2, 3, 4, 5), 3)

    try:
        resize_size_pixel_x = config.getint("Resize", "size_pixel_x")
    except:
        resize_size_pixel_x = "1024"
    dict_return["resize_size_pixel_x"] = resize_size_pixel_x

    try:
        resize_size_pixel_y = config.getint("Resize", "size_pixel_y")
    except:
        resize_size_pixel_y = "1024"
    dict_return["resize_size_pixel_y"] = resize_size_pixel_y

    try:
        resize_size_percent = config.getint("Resize", "size_percent")
    except:
        resize_size_percent = "25"
    dict_return["resize_size_percent"] = resize_size_percent

    return dict_return


def text(file_ini, fonts_dict, operating_system):
    """Text configuration"""

    # output dictionary
    dict_return = {}

    config = configparser.ConfigParser()
    config.read(file_ini, encoding="utf8")

    try:
        text_on = config.getint("Text", "on")
    except:
        text_on = "0"
    dict_return["img_text_on"] = entries.parse_list(text_on, (0, 1), 0)

    try:
        text_inout = config.getint("Text", "inout")
    except:
        text_inout = "1"
    dict_return["img_text_inout"] = entries.parse_list(text_inout, (0, 1), 1)

    try:
        text_string = config.get("Text", "text")
    except:
        text_string = ""
    dict_return["text_text"] = text_string

    try:
        text_gravity = config.get("Text", "gravity")
    except:
        text_gravity = "SE"
    dict_return["img_text_gravity"] = entries.parse_list(
        text_gravity, ("NW", "N", "NE", "W", "C", "E", "SW", "S", "SE"), "SE"
    )

    try:
        text_gravity_onoff = config.getint("Text", "gravity_onoff")
    except:
        text_gravity_onoff = "1"
    dict_return["img_text_gravity_onoff"] = entries.parse_list(
        text_gravity_onoff, (0, 1), 1
    )

    if operating_system == 'Windows':
        default_font = "Arial"
    elif operating_system == 'MACOS':
        default_font = "Helvetica"
    else:
        default_font = "DejaVu-Sans"
    try:
        text_font = config.get("Text", "font")
    except:
        text_font = default_font
    dict_return["text_font"] = entries.parse_list(text_font, fonts_dict, default_font)

    try:
        text_size = str(config.getint("Text", "size"))
    except:
        text_size = 12
    dict_return["text_size"] = text_size

    try:
        text_x = config.getint("Text", "x")
    except:
        text_x = "5"
    dict_return["text_x"] = text_x

    try:
        text_y = config.getint("Text", "y")
    except:
        text_y = "5"
    dict_return["text_y"] = text_y

    try:
        text_color = config.get("Text", "color")
    except:
        text_color = "#FFFFFF"
    dict_return["text_color"] = entries.parse_color(text_color, "#FFFFFF")

    try:
        text_box = config.getint("Text", "box")
    except:
        text_box = 0
    dict_return["text_box"] = entries.parse_list(text_box, (0, 1), 0)

    try:
        text_box_color = config.get("Text", "box_color")
    except:
        text_box_color = "#000000"
    dict_return["text_box_color"] = entries.parse_color(text_box_color, "#000000")

    try:
        text_rotate = config.getint("Text", "text_rotate")
    except:
        text_rotate = "0"
    dict_return["text_rotate"] = entries.parse_list(
        text_rotate, (-1, 0, 90, 180, 270), 0
    )

    try:
        rotate_own = config.getint("Text", "text_rotate_own")
    except:
        rotate_own = "0"
    dict_return["text_rotate_own"] = str(common.empty(rotate_own))

    try:
        text_arrow = config.getint("Text", "text_arrow")
    except:
        text_arrow = "0"
    dict_return["text_arrow"] = entries.parse_list(
        text_arrow, (0, 1), 0
    )

    return dict_return


def rotate(file_ini):
    """Rotate configuration"""

    # output dictionary
    dict_return = {}

    config = configparser.ConfigParser()
    config.read(file_ini, encoding="utf8")

    try:
        rotate_on = config.getint("Rotate", "on")
    except:
        rotate_on = "0"
    dict_return["img_rotate_on"] = entries.parse_list(rotate_on, (0, 1), 0)

    try:
        rotate_variant = config.getint("Rotate", "rotate")
    except:
        rotate_variant = "90"
    dict_return["img_rotate"] = entries.parse_list(rotate_variant, (0, 90, 180, 270), 0)

    try:
        rotate_own = config.getint("Rotate", "own")
    except:
        rotate_own = "0"
    dict_return["img_rotate_own"] = str(common.empty(rotate_own))

    try:
        rotate_color = config.get("Rotate", "color")
    except:
        rotate_color = "#FFFFFF"
    dict_return["img_rotate_color"] = entries.parse_color(rotate_color, "#FFFFFF")

    return dict_return


def crop(file_ini):
    """Crop configuration"""

    # output dictionary
    dict_return = {}

    config = configparser.ConfigParser()
    config.read(file_ini, encoding="utf8")

    try:
        crop_on = config.getint("Crop", "on")
    except:
        crop_on = "0"
    dict_return["img_crop_on"] = entries.parse_list(crop_on, (0, 1), 0)

    try:
        crop_variant = config.getint("Crop", "crop")
    except:
        crop_variant = "1"
    dict_return["img_crop"] = entries.parse_list(crop_variant, (1, 2, 3), 1)

    try:
        crop_1_x1 = config.getint("Crop", "1_x1")
    except:
        crop_1_x1 = "0"
    dict_return["crop_1_x1"] = crop_1_x1

    try:
        crop_1_y1 = config.getint("Crop", "1_y1")
    except:
        crop_1_y1 = "0"
    dict_return["crop_1_y1"] = crop_1_y1

    try:
        crop_1_x2 = config.getint("Crop", "1_x2")
    except:
        crop_1_x2 = "0"
    dict_return["crop_1_x2"] = crop_1_x2

    try:
        crop_1_y2 = config.getint("Crop", "1_y2")
    except:
        crop_1_y2 = "0"
    dict_return["crop_1_y2"] = crop_1_y2

    try:
        crop_2_x1 = config.getint("Crop", "2_x1")
    except:
        crop_2_x1 = "0"
    dict_return["crop_2_x1"] = crop_2_x1

    try:
        crop_2_y1 = config.getint("Crop", "2_y1")
    except:
        crop_2_y1 = "0"
    dict_return["crop_2_y1"] = crop_2_y1

    try:
        crop_2_width = config.getint("Crop", "2_width")
    except:
        crop_2_width = "0"
    dict_return["crop_2_width"] = crop_2_width

    try:
        crop_2_height = config.getint("Crop", "2_height")
    except:
        crop_2_height = "0"
    dict_return["crop_2_height"] = crop_2_height

    try:
        crop_3_dx = config.getint("Crop", "3_dx")
    except:
        crop_3_dx = "0"
    dict_return["crop_3_dx"] = crop_3_dx

    try:
        crop_3_dy = config.getint("Crop", "3_dy")
    except:
        crop_3_dy = "0"
    dict_return["crop_3_dy"] = crop_3_dy

    try:
        crop_3_width = config.getint("Crop", "3_width")
    except:
        crop_3_width = "0"
    dict_return["crop_3_width"] = crop_3_width

    try:
        crop_3_height = config.getint("Crop", "3_height")
    except:
        crop_3_height = "0"
    dict_return["crop_3_height"] = crop_3_height

    try:
        crop_gravity = config.getint("Crop", "gravity")
    except:
        crop_gravity = "C"
    dict_return["img_crop_gravity"] = entries.parse_list(
        crop_gravity, ("NW", "N", "NE", "W", "C", "E", "SW", "S", "SE"), "SE"
    )

    return dict_return


def border(file_ini):
    """Border congiguration"""

    # output dictionary
    dict_return = {}

    config = configparser.ConfigParser()
    config.read(file_ini, encoding="utf8")

    try:
        border_on = config.getint("Border", "on")
    except:
        border_on = "0"
    dict_return["img_border_on"] = entries.parse_list(border_on, (0, 1), 0)

    try:
        border_color = config.get("Border", "color")
    except:
        border_color = "#FFFFFF"
    dict_return["img_border_color"] = entries.parse_color(border_color, "#FFFFFF")

    try:
        border_x = config.getint("Border", "size_x")
    except:
        border_x = "10"
    dict_return["img_border_size_x"] = border_x

    try:
        border_y = config.getint("Border", "size_y")
    except:
        border_y = "10"
    dict_return["img_border_size_y"] = border_y

    return dict_return


def vignette(file_ini):
    """Vignette congiguration"""

    # output dictionary
    dict_return = {}

    config = configparser.ConfigParser()
    config.read(file_ini, encoding="utf8")

    try:
        vignette_on = config.getint("Vignette", "on")
    except:
        vignette_on = "0"
    dict_return["on"] = entries.parse_list(vignette_on, (0, 1), 0)

    try:
        color = config.get("Vignette", "color")
    except:
        color = "#FFFFFF"
    dict_return["color"] = entries.parse_color(color, "#FFFFFF")

    try:
        dx = config.getint("Vignette", "dx")
    except:
        dx = "0"
    dict_return["dx"] = dx

    try:
        dy = config.getint("Vignette", "dy")
    except:
        dy = "0"
    dict_return["dy"] = dy

    try:
        radius = config.getint("Vignette", "radius")
    except:
        radius = "0"
    dict_return["radius"] = radius

    try:
        sigma = config.getint("Vignette", "sigma")
    except:
        sigma = "0"
    dict_return["sigma"] = sigma

    return dict_return


def colors(file_ini):
    """Color configuration"""

    # output dictionary
    dict_return = {}

    config = configparser.ConfigParser()
    config.read(file_ini, encoding="utf8")

    try:
        color_on = config.getint("Color", "on")
    except:
        color_on = "0"
    dict_return["color_on"] = entries.parse_list(color_on, (0, 1), 0)

    try:
        black_white = config.getint("Color", "black-white")
    except:
        black_white = "1"
    dict_return["black_white"] = entries.parse_list(black_white, (1, 2), 1)

    try:
        bw_sepia = config.getint("Color", "sepia")
    except:
        bw_sepia = "95"
    dict_return["sepia"] = bw_sepia

    return dict_return


def normalize(file_ini, channels):
    """Normalize"""

    # output dictionary
    dict_return = {}

    config = configparser.ConfigParser()
    config.read(file_ini, encoding="utf8")

    try:
        normalize_on = config.getint("Normalize", "on")
    except:
        normalize_on = 0
    dict_return["normalize_on"] = entries.parse_list(normalize_on, (0, 1), 0)

    try:
        normalize_variant = config.getint("Normalize", "normalize")
    except:
        normalize_variant = 1
    dict_return["normalize"] = entries.parse_list(normalize_variant, (1, 2), 1)

    try:
        normalize_type = config.get("Normalize", "channel")
    except:
        normalize_type = "None"
    dict_return["channel"] = entries.parse_list(normalize_type, channels, "None")

    return dict_return


def contrast(file_ini, contrast_selection_valid_list):
    """Contrast configuration"""

    # output dictionary
    dict_return = {}

    config = configparser.ConfigParser()
    config.read(file_ini, encoding="utf8")

    try:
        contrast_on = config.getint("Contrast", "on")
    except:
        contrast_on = "0"
    dict_return["contrast_on"] = entries.parse_list(contrast_on, (0, 1), 0)

    try:
        contrast_variant = config.getint("Contrast", "contrast")
    except:
        contrast_variant = "1"
    dict_return["contrast"] = entries.parse_list(contrast_variant, (1, 2, 3), 1)

    try:
        contrast_selection = config.get("Contrast", "selection")
    except:
        contrast_selection = "0"
    dict_return["contrast_selection"] = entries.parse_list(
        contrast_selection, contrast_selection_valid_list, "+1"
    )

    try:
        contrast_stretch_1 = float(config.get("Contrast", "stretch1"))
    except:
        contrast_stretch_1 = "0.15"
    dict_return["contrast_stretch_1"] = contrast_stretch_1

    try:
        contrast_stretch_2 = float(config.get("Contrast", "stretch2"))
    except:
        contrast_stretch_2 = "0.05"
    dict_return["contrast_stretch_2"] = contrast_stretch_2

    return dict_return


def logo(file_ini):
    """Logo cofiguration"""

    # output dictionary
    dict_return = {}

    config = configparser.ConfigParser()
    config.read(file_ini, encoding="utf8")

    try:
        logo_on = config.getint("Logo", "on")
    except:
        logo_on = 0
    dict_return["img_logo_on"] = entries.parse_list(logo_on, (0, 1), 0)

    try:
        logo_filename = config.get("Logo", "logo")
    except:
        logo_filename = ""
    dict_return["logo_logo"] = logo_filename

    try:
        logo_gravity = config.get("Logo", "gravity")
    except:
        logo_gravity = "SE"
    dict_return["img_logo_gravity"] = entries.parse_list(
        logo_gravity, ("NW", "N", "NE", "W", "C", "E", "SW", "S", "SE"), "SE"
    )

    try:
        logo_width = config.getint("Logo", "width")
    except:
        logo_width = "100"
    dict_return["logo_width"] = logo_width

    try:
        logo_height = config.getint("Logo", "height")
    except:
        logo_height = "100"
    dict_return["logo_height"] = logo_height

    try:
        logo_dx = config.getint("Logo", "dx")
    except:
        logo_dx = "5"
    dict_return["logo_dx"] = logo_dx

    try:
        logo_dy = config.getint("Logo", "dy")
    except:
        logo_dy = "5"
    dict_return["logo_dy"] = logo_dy

    return dict_return


def mirror(file_ini):
    """Rotate configuration"""

    # output dictionary
    dict_return = {}

    config = configparser.ConfigParser()
    config.read(file_ini, encoding="utf8")

    try:
        mirror_on = config.getint("Mirror", "on")
    except:
        mirror_on = "0"
    dict_return["img_mirror_on"] = entries.parse_list(mirror_on, (0, 1), 0)

    try:
        flip = config.getint("Mirror", "flip")
    except:
        flip = "0"
    dict_return["img_mirror_flip"] = entries.parse_list(flip, (0, 1), 0)

    try:
        flop = config.getint("Mirror", "flop")
    except:
        flop = "0"
    dict_return["img_mirror_flop"] = entries.parse_list(flop, (0, 1), 0)

    return dict_return


def compose(file_ini, preview_size_list):
    """Compose congiguration"""

    # output dictionary
    dict_return = {}

    config = configparser.ConfigParser()
    config.read(file_ini, encoding="utf8")

    try:
        on = config.getint("Compose", "on")
    except:
        on = "0"
    dict_return["compose_on"] = entries.parse_list(on, (0, 1), 0)

    try:
        file = config.get("Compose", "filename")
    except:
        file = ""
    dict_return["compose_filename"] = file

    try:
        right = config.getint("Compose", "right")
    except:
        right = 0
    dict_return["compose_right"] = entries.parse_list(right, (0, 1), 0)

    try:
        autoresize = config.getint("Compose", "autoresize")
    except:
        autoresize = 1
    dict_return["compose_autoresize"] = entries.parse_list(autoresize, (0, 1), 1)

    try:
        color = config.get("Compose", "color")
    except:
        color = "#FFFFFF"
    dict_return["compose_color"] = entries.parse_color(color, "#FFFFFF")

    try:
        gravity = config.get("Compose", "gravity")
    except:
        gravity = "C"
    dict_return["compose_gravity"] = entries.parse_list(
        gravity, ("N", "W", "C", "E", "S"), "C"
    )

    try:
        preview = config.getint("Compose", "preview")
    except:
        preview = 400
    dict_return["preview"] = entries.parse_list(preview, preview_size_list, 400)
    return dict_return


# EOF



# -*- coding: utf-8 -*-

"""
Copyright (c) 2024 Tomasz Łuczak, TeaM-TL

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included
in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

From https://stackoverflow.com/questions/28774852/pypi-api-how-to-get-stable-package-version
"""

import json
import logging
import requests
import time

try:
    from packaging.version import parse
except ImportError:
    from pip._vendor.packaging.version import parse

module_logger = logging.getLogger(__name__)

URL_PATTERN = "https://pypi.org/pypi/{package}/json"


def get_version(url_pattern=URL_PATTERN):
    """Return version of package on pypi.python.org using json."""

    package = "FotoKilof"
    version = parse("0")
    try:
        req = requests.get(url_pattern.format(package=package), timeout=2.50)
        request_success = 1
    except:
        request_success = 0

    if request_success and req.status_code == requests.codes.ok:
        j = json.loads(req.text.encode(req.encoding))
        releases = j.get("releases", [])
        for release in releases:
            ver = parse(release)
            if not ver.is_prerelease:
                version = max(version, ver)
    return (request_success, str(version))


def check_version(current_version):
    """
    Check version of FotoKilof in PyPi
    return 1 - get new
    return 0 - there are no new
    """
    start_time = time.time()
    result = 0
    result_success, result_version = get_version()
    if result_success and result_version > current_version:
        result = 1
    module_logger.info("check_version: %ss", str(time.time() - start_time))
    return (result, result_version)



# -*- coding: utf-8 -*-

"""
Copyright (c) 2024 Tomasz Łuczak, TeaM-TL

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included
in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
"""

import check_new_version
import common
import gui
import version


def test_check_version():
    """tests for check_new_version"""
    assert check_new_version.check_version(version.__version__)[0] == 0


def test_gui_only_numbers():
    """test only numbers"""
    assert gui.only_numbers("1")
    # assert gui.only_numbers('a')


def test_gui_only_integer():
    """test only integer"""
    assert gui.only_integer(1)


def test_common_empty():
    """test empty"""
    assert common.empty("") == 0
    assert common.empty("11") == 11


def test_common_humansize():
    """test humansize"""
    assert common.humansize(100) == "100 B"
    assert common.humansize(2048) == "2 kB"


def test_common_crop_gravity():
    """test crom_gravity
    coordinates = (offset_x, offset_y, width, height, gravitation)
    """
    coordinates = (0, 0, 100, 100, "SE")
    assert common.crop_gravity(coordinates, 1000, 500) == (900, 400, 1000, 500)


def test_common_arrow_gravity():
    """test arrow gravity"""
    position = "C"
    length = 40
    dx = 200
    dy = 200

    for position in ("N", "S", "E", "W", "NW", "NE", "SW", "SE"):
        if position == "C":
            a, c, d, e = ((0, 0), (0, 0), (0, 0), (0, 0))
            x = 0
            y = 0
        elif position == "N":
            a, c, d, e, x, y = ((200, 240), (200, 200), (194, 213), (206, 213), 0, 40)
        elif position == "S":
            a, c, d, e, x, y = ((200, 160), (200, 200), (194, 187), (206, 187), 0, -40)
        elif position == "E":
            a, c, d, e, x, y = ((160, 200), (200, 200), (187, 194), (187, 206), -40 , 0)
        elif position == "W":
            a, c, d, e, x, y = ((240, 200), (200, 200), (213, 194), (213, 206), 40, 0)
        elif position == "NW":
            a, c, d, e, x, y = ((240, 240), (200, 200), (210, 220), (220, 210), 40, 40)
        elif position == "NE":
            a, c, d, e, x, y = ((160, 240), (200, 200), (190, 220), (180, 210), -40, 40)
        elif position == "SW":
            a, c, d, e, x, y = ((240, 160), (200, 200), (210, 180), (220, 190), 40, -40)
        elif position == "SE":
            a, c, d, e, x, y = ((160, 160), (200, 200), (190, 180), (180, 190), -40 , -40)

        print(position, a, c, d, e, x, y)
        assert common.arrow_gravity(position, length, dx, dy) == (
            a,
            c,
            d,
            e,
            x,
            y,
        )



# -*- coding: utf-8 -*-

"""
Copyright (c) 2019-2021 Tomasz Łuczak, TeaM-TL

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included
in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

module contains function for parsing GUI entries and INI entries:
- parse_list - validation entry if is on list
- parse_range - validation entry if is in range
- parse_color - validation entry if is a color specification
"""

import re


def parse_list(entry, valid, default):
    """
    parsing entries
    entry - value to check
    valid - valid range
    default - value if entry is invalid
    return - entry or default if entry is out of range
    """
    if entry in valid:
        result = entry
    else:
        result = default

    return result


def parse_range(entry, valid, default):
    """
    parsing entries
    entry - value to check
    default - value if entry is invalid
    valid - valid range
    return - entry or default if entry is out of range
    """

    if valid[0] <= entry <= valid[1]:
        result = entry
    else:
        result = default

    return result


def parse_color(entry, default):
    """
    parsing entries
    entry - value to check
    default - value if entry is invalid
    return - entru or default if entry is out of range
    """
    if re.match("^[#][0-9a-fA-F]{6}$", entry):
        result = entry
    else:
        result = default
    return result


# EOF



# -*- coding: utf-8 -*-
# pylint: disable=bare-except

"""
Copyright (c) 2019-2024 Tomasz Łuczak, TeaM-TL

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included
in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

module to run ImageMagick:
- magick - picture conversion
"""

import logging
import os

import common

module_logger = logging.getLogger(__name__)


def magick(cmd, file_in, file_out, command, operating_system):
    """
    run imagemagick command.
    cmd - command for imagemagick
    file_in - fullname picture for processing
    file_out - fullname output picture
    command:
      convert, mogrify, composite, import - ImageMagick
    """
    result = None
    if cmd != "":
        if file_in is not None:
            file_in = common.spacja(file_in, operating_system)
            file_out = common.spacja(file_out, operating_system)
            if operating_system == 'Windows':
                prefix_cmd = "magick.exe "
            else:
                prefix_cmd = ""
            command_exec = (
                prefix_cmd + command + " " + file_in + " " + cmd + " " + file_out
            )
            module_logger.info("Execute: %s", command_exec)
            try:
                os.system(command_exec)
            except:
                module_logger.error("Errot in imagick: %s", command_exec)
                result = None
            else:
                result = "OK"
        else:
            module_logger.warning("imagick: No file for imagick")
            result = None
    else:
        result = None
    return result


# EOF



# -*- coding: utf-8 -*-
# pylint: disable=too-many-branches

"""
Copyright (c) 2019-2024 Tomasz Łuczak, TeaM-TL

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included
in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.


- out_full_filename - prepare output file for conversion
"""

import os

import logging

module_logger = logging.getLogger(__name__)


def out_full_filename(file_in, destination, extension):
    """
    Input:
        file_in - original file for processing
        destination - output directory
        extension - extension of result file, for change format (jpg->png)
    Output:
        file_out - fullname file for processing in destination
    """
    result = "OK"  # initial value
    if file_in is not None:
        if os.path.isfile(file_in):
            # making output diretory if not exist
            out_dir = os.path.join(os.path.dirname(file_in), destination)
            if os.path.isdir(out_dir) is False:
                try:
                    os.mkdir(out_dir)
                except FileExistsError:
                    module_logger.error("pre_imagick: FileExistsError %s", out_dir)
                except FileNotFoundError:
                    try:
                        os.mkdir(os.path.dirname(out_dir))
                    except FileNotFoundError:
                        module_logger.error(
                            "pre_imagick: Cannot make directory for output pictures %s",
                            os.path.dirname(out_dir),
                        )
                        result = None
                    except:
                        module_logger.error(
                            "pre_imagick: other problem to create %s",
                            os.path.dirname(out_dir),
                        )
                        result = None
                    if result == "OK":
                        try:
                            os.mkdir(out_dir)
                        except FileExistsError:
                            module_logger.error("pre_imagick: FileExistsError %s", out_dir)
                        except FileNotFoundError:
                            module_logger.error(
                                "pre_imagick: FileExistsError %s",
                                os.path.dirname(out_dir),
                            )
                            result = None
                        except:
                            module_logger.error(
                                "pre_imagick: other problem to create %s", out_dir
                            )
                            result = None
                except:
                    module_logger.error("pre_imagick: other problem to create %s", out_dir)
                    result = None

        else:
            result = None
    else:
        result = None

    if result == "OK":
        # preparing output filename
        file_in_without_ext = os.path.splitext(file_in)
        file_out = os.path.join(
            out_dir, os.path.basename(file_in_without_ext[0] + extension)
        )
    else:
        file_out = None
    return file_out


# EOF



# -*- coding: utf-8 -*-
"""
Copyright (c) 2019-2023 Tomasz Łuczak, TeaM-TL

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included
in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

function for GUI:
- copy_to_clipboard - copy result into clipboard (only Windows)
- only_numbers - validate only natural values
- only_integer - validate integer values
"""

from io import BytesIO
import logging
import platform
import re
from PIL import Image

import common

if platform.system() == "Windows":
    import win32clipboard
elif platform.system() == "Darwin":
    import subprocess


module_logger = logging.getLogger(__name__)


def copy_to_clipboard(file_in, operating_system):
    """
    Copy results into clipboard for Windows
    https://stackoverflow.com/questions/34322132/copy-image-to-clipboard
    Copy results into clipboard for Macos
    https://stackoverflow.com/questions/54008175/copy-an-image-to-macos-clipboard-using-python?rq=4
    debug needed!
    """
    if operating_system == "Windows":
        # Create an in-memory file-like object
        image_buffer = BytesIO()
        image = Image.open(common.spacja(file_in, operating_system))
        image.convert("RGB").save(image_buffer, "BMP")
        data = image_buffer.getvalue()[14:]

        win32clipboard.OpenClipboard()
        win32clipboard.EmptyClipboard()
        win32clipboard.SetClipboardData(win32clipboard.CF_DIB, data)
        win32clipboard.CloseClipboard()
        image_buffer.close()
    elif operating_system == "MACOS":
        try:
            subprocess.run(
                [
                    "osascript",
                    "-e",
                    'set the clipboard to (read (POSIX file "'
                    + file_in
                    + '") as JPEG picture)',
                ]
            )
            module_logger.debug(
                "Successful copied result into clipboard under MacOS: %s", file_in
            )
        except:
            module_logger.debug(
                "Failed copied result into clipboard under MacOS: %s", file_in
            )


def only_numbers(char):
    """Validation entry widgets: only digits"""
    return char.isdigit()


def only_integer(char):
    """Validation entry widgets: only digits"""
    if re.match("^[-]{0,1}[0-9]{,6}$", str(char)):
        result = True
    else:
        result = False
    return result


# EOF



# -*- coding: utf-8 -*-
# pylint: disable=bare-except

"""
Copyright (c) 2024 Tomasz Łuczak, TeaM-TL

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included
in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

Info
- fonts_list - get list of available fonts
Common
- make_clone - open origal picture and make clone for processing
- save_close_clone - save clone into file and close clone
- gravitation - translate eg. NS to Northsouth as Wand-py expect
Converters
- pip - picture in picture, for inserting logo
- rotate - rotate picture
- mirror - mirroring picture
- border - add border to picture
- text - add text into picture
- bw - black and white or sepia
- resize - resize picture
- normalize - normalize levels
- contrast - modify contrast
- crop - crop picture
- vignete - add vignete into picture
- compose - join two pictures
- preview - preview done by Pillow
"""

import logging
import tempfile
import os
import os.path
from PIL import Image, ImageDraw, ImageOps, ImageFont

from find_system_fonts_filename import (
    get_system_fonts_filename,
    FindSystemFontsFilenameException,
)

# my modules
import common

module_logger = logging.getLogger(__name__)


# ------------------------------------ Info
def version():
    """version of PIL"""
    return Image.__version__


def fonts_list():
    """list of available fonts"""
    try:
        fonts_filename = get_system_fonts_filename()
        module_logger.debug("Get fonts list from system")
    except FindSystemFontsFilenameException:
        # Deal with the exception
        module_logger.error("Errort to get fonts list from system")
        fonts_filename = None
    if fonts_filename is not None:
        result = []
        for item in fonts_filename:
            result.append(item)
        result.sort()
    else:
        result = "Arial"
    return result


# ------------------------------------ Common
def make_clone(file_to_clone, color=None):
    """open picture and make clone for processing"""
    if len(file_to_clone) > 0:
        with Image.open(file_to_clone) as image:
            result = image.copy()
    else:
        result = None
    return result


def save_close_clone(clone, file_out, ppm=0, exif=0):
    """save and close clone after processing"""
    if clone is None:
        module_logger.error(" Clone for %s is None", file_out)
    else:
        # if not exif:
        #     clone.strip()
        module_logger.debug(" Save file: %s", file_out)
        clone.save(file_out)
        clone.close()


def get_image_size(filename):
    """
    identify width and height of picture
    input: file name
    output: size (width, height)
    """
    size = (0, 0)
    if filename is not None:
        if os.path.isfile(filename):
            try:
                with Image.open(filename) as image:
                    size = image.size
            except:
                module_logger.error(" Error read file: %s", filename)
    module_logger.debug("get_image_size: %s, %s", filename, str(size))
    return size


def gravitation(gravity):
    """translate gravitation name from Tk to Pillow specification"""

    if gravity == "N":
        result = "ma"
    if gravity == "NW":
        result = "la"
    if gravity == "NE":
        result = "ra"
    if gravity == "W":
        result = "lm"
    if gravity == "C":
        result = "mm"
    if gravity == "E":
        result = "rm"
    if gravity == "SW":
        result = "ld"
    if gravity == "S":
        result = "ld"
    if gravity == "SE":
        result = "rd"
    if gravity == "0":
        result = "0"

    return result


# ------------------------------------ Converters
def pip(clone, logo, logo_data, image_height, image_width):
    """put picture on picture
    clone - clone of image for processing
    logo - filename of logo
    logo_data = offset_x, offset_y, width, height, gravitation
    original image size: image_height, image_width
    """
    if len(logo):
        with Image(filename=logo) as logo_img:
            with Drawing() as draw:
                position = common.crop_gravity(logo_data, image_height, image_width)
                draw.composite(
                    operator="over",
                    left=common.empty(position[0]),
                    top=common.empty(position[1]),
                    width=common.empty(logo_data[2]),
                    height=common.empty(logo_data[3]),
                    image=logo_img,
                )
                draw(clone)
    module_logger.debug(" Conversion: logo")


def rotate(clone, angle, color, own):
    """rotate"""
    if angle == 0:
        angle = common.empty(own)
        if angle == 0:
            color = None
    else:
        color = None
    result = clone.rotate(angle=angle, fillcolor=color, expand=True)
    module_logger.debug(" Conversion: rotate %s", str(angle))
    return result


def mirror(clone, flip, flop):
    """mirror: flip and flop"""
    result = clone
    if flip:
        # result = clone.transpose(Image.Transpose.FLIP_TOP_BOTTOM)
        result = ImageOps.flip(clone)
    if flop:
        # result = result.transpose(Image.Transpose.FLIP_LEFT_RIGHT)
        result = ImageOps.mirror(result)
    module_logger.debug(" Conversion: mirror")
    return result


def border(clone, color, x, y):
    """border: color, x, y"""
    result = ImageOps.expand(clone, (int(x), int(y)), color)
    module_logger.debug("Conversion: border")
    return result


def text(convert_data):
    """add text into picture"""
    clone = convert_data[0]
    in_out = convert_data[1]
    own = convert_data[2]
    angle = convert_data[3]
    angle = 0
    text_color = convert_data[4]
    font = convert_data[5]
    text_size = int(convert_data[6])
    gravity_onoff = convert_data[7]
    gravity = convert_data[8]
    box = convert_data[9]
    box = 0
    box_color = convert_data[10]
    text_x = int(common.empty(convert_data[11]))
    text_y = int(common.empty(convert_data[12]))
    text_string = convert_data[13]
    arrow = convert_data[14]

    image_width, image_height = clone.size
    font = ImageFont.truetype(font, text_size)

    if len(text_string):
        if in_out == 0:
            # inside
            if gravity_onoff == 0:
                draw_gravity = "lt"
            else:
                if gravity == "NW":
                    draw_gravity = "lt"
                elif gravity == "N":
                    draw_gravity = "mt"
                    text_x += image_width / 2
                elif gravity == "NE":
                    draw_gravity = "rt"
                    text_x = image_width - text_x
                elif gravity == "W":
                    draw_gravity = "lm"
                    text_y += image_height / 2
                elif gravity == "C":
                    draw_gravity = "mm"
                    text_x += image_width / 2
                    text_y += image_height / 2
                elif gravity == "E":
                    draw_gravity = "rm"
                    text_x = image_width - text_x
                    text_y += image_height / 2
                elif gravity == "SW":
                    draw_gravity = "lb"
                    text_y = image_height - text_y
                elif gravity == "S":
                    draw_gravity = "mb"
                    text_x += image_width / 2
                    text_y = image_height - text_y
                elif gravity == "SE":
                    draw_gravity = "rb"
                    text_x = image_width - text_x
                    text_y = image_height - text_y
            draw_text = ImageDraw.Draw(clone)
            if arrow:
                if gravity_onoff == 0:
                    gravity = "NW"
                a, c, d, e, offset_x, offset_y = common.arrow_gravity(gravity, text_size, text_x, text_y)
                if gravity != "C":
                    draw_text.line([a, c], fill=text_color, width=2)
                    draw_text.line([d, c], fill=text_color, width=2)
                    draw_text.line([e, c], fill=text_color, width=2)
            else:
                offset_x = 0
                offset_y = 0
            draw_text.text(
                (text_x + offset_x, text_y + offset_y),
                text_string,
                fill=text_color,
                font=font,
                anchor=draw_gravity,
            )
            result = clone
            module_logger.debug(" Conversion: text inside")
        else:
            # outside
            left, top, right, bottom = font.getbbox(text_string)
            if gravity == "W":
                text_x = 0
                draw_gravity = "lt"
            elif gravity == "C":
                text_x = image_width / 2
                draw_gravity = "mt"
            elif gravity == "E":
                text_x = image_width
                draw_gravity = "rt"
            else:
                text_x = 0
                draw_gravity = "lt"
            image_text = Image.new("RGB", (image_width, bottom), box_color)
            draw_outside_text = ImageDraw.Draw(image_text)
            draw_outside_text.text(
                (text_x, top / 2),
                text_string,
                fill=text_color,
                font=font,
                anchor=draw_gravity,
            )
            image_outside = Image.new("RGB", (image_width, image_height + bottom))
            image_outside.paste(clone)
            image_outside.paste(image_text, (0, image_height))
            result = image_outside
            module_logger.debug(" Conversion: text outside")

        module_logger.debug(" Conversion: text")
    return result


def bw(clone, bw_variant, sepia):
    """black and white or sepia"""
    if bw_variant == 1:
        # black-white
        result = ImageOps.grayscale(clone)
    else:
        module_logger.warning(
            "Black-white/sepia not available for PILLOW, install ImageMagick"
        )
        # sepia
        # clone.sepia_tone(threshold=common.empty(sepia) / 100)
    module_logger.debug(" Conversion: black-white/sepia %s", str(bw_variant))
    return result


def resize(clone, size):
    """resize picture"""
    image_width, image_height = clone.size
    if "x" in size:
        width, height = size.split("x")
        width = int(width)
        height = int(height)

        if width > height:
            max_size = width
        else:
            max_size = height

        ratio = 1
        if width > height:
            final_width, final_height = (
                max_size,
                int(image_height * (max_size / image_width)),
            )
            if final_height > height:
                ratio = height / final_height
        elif width < height:
            final_width, final_height = (
                int(image_width * (max_size / image_height)),
                max_size,
            )
            if final_width > width:
                ratio = width / final_width
        else:
            final_width, final_height = (max_size, max_size)
        final_width = int(final_width * ratio)
        final_height = int(final_height * ratio)
    else:
        max_size = int(size.split("%")[0])
        final_width = max_size * image_width / 100
        final_height = max_size * image_height / 100

    result = clone.resize((int(final_width), int(final_height)))
    module_logger.debug(
        " Conversion: resize %s x %s", str(final_width), str(final_height)
    )
    return result


def normalize(clone, normalize_variant, channel):
    """normalize levels of colors"""
    if normalize_variant == 1:
        result = ImageOps.autocontrast(clone)
        # if channel != "None":
        #     clone.alpha_channel = True
        #     clone.normalize(channel=channel)
        # else:
        #     clone.normalize()
    else:
        result = ImageOps.equalize(clone)
    module_logger.debug(" Conversion: normalize %s", str(normalize_variant))
    return result


def contrast(clone, contrast_variant, selection, black, white):
    """normalize levels of colors"""
    result = None
    if int(contrast_variant) == 1:
        if float(black) > 100:
            black = 100
        if float(white) > 100:
            white = 100
        # clone.contrast_stretch(black_point=float(black), white_point=float(white))
        result = ImageOps.autocontrast(clone, (float(black), float(white)))
    # else:
    #     if int(selection) != 0:
    #         if int(selection) > 0:
    #             sharpen = True
    #         else:
    #             sharpen = False
    #         iteration = 0
    #         while iteration < abs(int(selection)):
    #             iteration += 1
    #             clone.contrast(sharpen=sharpen)
    module_logger.debug(" Conversion: contrast %s", str(contrast_variant))
    return result


def crop(file_in, clone, crop_variant, gravity, entries):
    """
    crop picture
    entries are as dictionary
    """
    image_size = get_image_size(file_in)

    if crop_variant == 1:
        if (entries["one_x1"] < entries["one_x2"]) and (
            entries["one_y1"] < entries["one_y2"]
        ):
            if entries["one_x2"] > image_size[0]:
                entries["one_x2"] = image_size[0]
            if entries["one_y2"] > image_size[1]:
                entries["one_y2"] = image_size[1]
            left = entries["one_x1"]
            top = entries["one_y1"]
            right = entries["one_x2"]
            bottom = entries["one_y2"]
            result = clone.crop((left, top, right, bottom))
    if crop_variant == 2:
        if (entries["two_width"] > 0) and (entries["two_height"] > 0):
            left = entries["two_x1"]
            top = entries["two_y1"]
            right = left + entries["two_width"]
            bottom = top + entries["two_height"]
            result = clone.crop((left, top, right, bottom))
    if crop_variant == 3:
        if (entries["three_width"] > 0) and (entries["three_height"] > 0):
            clone = clone.crop(
                common.crop_gravity(
                    (
                        entries["three_dx"],
                        entries["three_dy"],
                        entries["three_width"],
                        entries["three_height"],
                        gravity,
                    ),
                    image_size[0],
                    image_size[1],
                )
            )
        result = clone
    module_logger.debug(" Conversion: crop %s", str(crop_variant))
    return result


def vignette(clone, dx, dy, radius, sigma):
    """add vignette into picture
    clone - clone of image for processing
    dx, dy - offset from border
    radius - radius of Gaussian blur
    sigma - standard deviation for Gaussian blur
    color - color of corners
    """
    # clone.vignette(
    #     radius=common.empty(radius),
    #     sigma=common.empty(sigma),
    #     x=common.empty(dx),
    #     y=common.empty(dy),
    # )
    # module_logger.debug(" Conversion: vigette")
    module_logger.warning("vigette not available for PILLOW, install ImageMagick")


def compose(clone, compose_file, right, autoresize, color, gravity):
    """join two pictures
    clone - clone of image for processing
    compose_file - file to join
    right - join on right or bottom side
    autoresize - autoresize picture or not
    color - color to fill gap if no autoresize
    gravity - position if no autoresize
    """
    result = None
    if os.path.exists(compose_file):
        clone_width, clone_height = clone.size
        with Image.open(compose_file) as compose_image:
            compose_image_width, compose_image_height = compose_image.size
            if right:
                if clone_height >= compose_image_height:
                    new_image_height = clone_height
                else:
                    new_image_height = compose_image_height
                if autoresize:
                    resize_width = int(
                        compose_image_width * clone_height / compose_image_height
                    )
                    resize_height = clone_height
                    new_image_width = clone_width + resize_width
                else:
                    new_image_width = clone_width + compose_image_width
                # for no autoresize
                position_x1 = 0
                position_x2 = clone_width
                if clone_height >= compose_image_height:
                    # orig > compose
                    position_y1 = 0
                    if gravity == "N":
                        position_y2 = 0
                    elif gravity == "S":
                        position_y2 = int(new_image_height - compose_image_height)
                    else:
                        position_y2 = int(
                            new_image_height / 2 - compose_image_height / 2
                        )
                    if autoresize:
                        position_y2 = 0
                else:
                    # orig < compose
                    position_y2 = 0
                    if gravity == "N":
                        position_y1 = 0
                    elif gravity == "S":
                        position_y1 = int(new_image_height - clone_height)
                    else:
                        position_y1 = int(new_image_height / 2 - clone_height / 2)
                    if autoresize:
                        position_y1 = 0
            else:
                # for canvas
                if clone_width >= compose_image_width:
                    new_image_width = clone_width
                else:
                    new_image_width = compose_image_width
                if autoresize:
                    resize_width = clone_width
                    resize_height = int(
                        compose_image_height * clone_width / compose_image_width
                    )
                    new_image_height = int(clone_height + resize_height)
                else:
                    new_image_height = int(clone_height + compose_image_height)
                # for no autoresize
                position_y1 = 0
                position_y2 = clone_height
                if clone_width >= compose_image_width:
                    # orig > compose
                    position_x1 = 0
                    if gravity == "W":
                        position_x2 = 0
                    elif gravity == "E":
                        position_x2 = int(new_image_width - compose_image_width)
                    else:
                        position_x2 = int(new_image_width / 2 - compose_image_width / 2)
                    if autoresize:
                        position_x2 = 0
                else:
                    # orig < compose
                    position_x2 = 0
                    if gravity == "W":
                        position_x1 = 0
                    elif gravity == "E":
                        position_x1 = int(new_image_width - clone_width)
                    else:
                        position_x1 = int(new_image_width / 2 - clone_width / 2)
                    if autoresize:
                        position_x1 = 0

            if autoresize:
                compose_image = compose_image.resize((resize_width, resize_height))

            image_new = Image.new("RGB", (new_image_width, new_image_height), color)
            image_new.paste(clone, (position_x1, position_y1))
            image_new.paste(compose_image, (position_x2, position_y2))
            result = image_new
    else:
        module_logger.warning(" Conversion: compose - missing file to compose")
    module_logger.debug(" Conversion: compose")
    return result


# ------------------------------------ Preview
def preview(file_in, max_size, operating_system, coord=""):
    """
    preview generation by Pillow
    file_in - fullname image file
    max_size - required size of image
    operating_system - operating system: Windows, MACOS, UNIX
    coord - coordinates for crop
    --
    return:
    - filename - path to PPM file
    - file size
    - width and height
    """

    result = {
        "filename": None,
        "size": "0",
        "width": "0",
        "height": "0",
        "preview_width": "0",
        "preview_height": "0",
    }

    if file_in is not None:
        if os.path.isfile(file_in):
            filesize = common.humansize(os.path.getsize(file_in))
            clone = make_clone(file_in)
            width, height = clone.size
            if width > height:
                preview_width, preview_height = (
                    max_size,
                    int(height * (max_size / width)),
                )
            elif width < height:
                preview_width, preview_height = (
                    int(width * (max_size / height)),
                    max_size,
                )
            else:
                preview_width, preview_height = (max_size, max_size)
            size = (preview_width, preview_height)
            clone = clone.resize(size)
            # write crop if coordinates are given
            if len(coord) == 4:
                left_top = (coord[0], coord[1])
                right_bottom = (coord[2], coord[3])
                rectangle = (left_top, right_bottom)
                outline_color = "#FFFF00"
                if coord[0] < coord[2] and coord[1] < coord[3]:
                    draw = ImageDraw.Draw(clone)
                    draw.rectangle(rectangle, outline=outline_color, fill=None, width=2)
            preview_width, preview_height = clone.size
            file_preview = os.path.join(tempfile.gettempdir(), "fotokilof_preview.ppm")
            save_close_clone(clone, file_preview)
            result = {
                "filename": common.spacja(file_preview, operating_system),
                "size": filesize,
                "width": str(width),
                "height": str(height),
                "preview_width": str(preview_width),
                "preview_height": str(preview_height),
            }
            module_logger.debug("preview: %s", str(result))
    return result


# EOF



# -*- coding: utf-8 -*-

"""
Copyright (c) 2019-2024 Tomasz Łuczak, TeaM-TL

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included
in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

module contains common functions:
- resize_subdir - calculation of directory name and converion argument for resize
- empty - convert empty string into O, no empty into int
- humansize - converts B to kB or MB
- mouse_crop_calculation - recalculation pixels from previev original image
- spacja - escaping space and special char in pathname
- crop_gravity - convert coordinates for crop3 and logo position
- list_of_images - sorted list images in cwd
- file_from_list_of_images - return filename from file_list depends of request
- arrow_gravity - calculate coordinates if draw arrow
"""

import fnmatch
import logging
from pathlib import PurePosixPath, PureWindowsPath
import os
import os.path

module_logger = logging.getLogger(__name__)


def resize_subdir(resize_vatiant, pixel_x, pixel_y, percent):
    """prepare name for subdir and command for resize"""
    if resize_vatiant == 1:
        command = str(pixel_x) + "x" + str(pixel_y)
        sub_dir = str(pixel_x) + "x" + str(pixel_y)
    elif resize_vatiant == 2:
        if percent > 100:
            percent = 100
        if percent == 0:
            percent = 1
        command = str(percent) + "%"
        sub_dir = str(percent)
    elif resize_vatiant == 3:
        command = "1920x1080"
        sub_dir = "1920x1080"
    elif resize_vatiant == 4:
        command = "2048x1556"
        sub_dir = "2048x1556"
    elif resize_vatiant == 5:
        command = "4096x3112"
        sub_dir = "4096x3112"
    return (sub_dir, command)


def empty(value):
    """
    convert empty string into 0
    non empty into int
    """
    if value == "":
        result = 0
    else:
        result = int(value)
    return result


def humansize(nbytes):
    """
    convert size in Byte into human readable: kB, MB, GB
    https://stackoverflow.com/questions/14996453/python-libraries-to-calculate-human-readable-filesize-from-bytes
    """
    suffixes = ["B", "kB", "MB", "GB", "TB", "PB"]
    i = 0
    while nbytes >= 1024 and i < len(suffixes) - 1:
        nbytes /= 1024.0
        i += 1
    value = ("%.2f" % nbytes).rstrip("0").rstrip(".")
    return "%s %s" % (value, suffixes[i])


def mouse_crop_calculation(x_orig, y_orig, size):
    """recalculation pixels from previev original image"""

    # x_max, y_max - size of preview, we know max: PREVIEW
    if x_orig > y_orig:
        # preview pixels, calculation of y_max preview
        x_max = size
        y_max = x_max * y_orig / x_orig
    elif y_orig > x_orig:
        # calculation of x_max
        y_max = size
        x_max = y_max * x_orig / y_orig
    elif y_orig == x_orig:
        x_max = size
        y_max = size

    dict_return = {}
    dict_return["x_max"] = x_max
    dict_return["y_max"] = y_max
    dict_return["x_orig"] = x_orig
    dict_return["y_orig"] = y_orig
    return dict_return


def spacja(file_path, operating_system):
    """escaping space and special char in pathname"""
    if len(file_path):
        if operating_system == "Windows":
            result = PureWindowsPath(os.path.normpath(file_path))
        else:
            result = PurePosixPath(os.path.normpath(file_path))
    else:
        result = None
    return result


def crop_gravity(coordinates, x_max, y_max):
    """
    convert corrdinates from crop3:
    offset_x, offset_y, width, height, gravitation
    original image size:
    x_max, y_max
    return coordinates for drawing crop: x0, y0, x1, y1
    """

    offset_x, offset_y, width, height, gravitation = coordinates

    if gravitation == "NW":
        x0 = offset_x
        y0 = offset_y
        x1 = x0 + width
        y1 = y0 + height
    elif gravitation == "N":
        x0 = x_max / 2 - width / 2
        y0 = offset_y
        x1 = x_max / 2 + width / 2
        y1 = y0 + height
    elif gravitation == "NE":
        x0 = x_max - width - offset_x
        y0 = offset_y
        x1 = x_max - offset_x
        y1 = y0 + height
    elif gravitation == "W":
        x0 = offset_x
        y0 = y_max / 2 - height / 2
        x1 = x0 + width
        y1 = y_max / 2 + height / 2
    elif gravitation == "C":
        x0 = x_max / 2 - width / 2 + offset_x
        y0 = y_max / 2 - height / 2
        x1 = x_max / 2 + width / 2 + offset_x
        y1 = y_max / 2 + height / 2
    elif gravitation == "E":
        x0 = x_max - width - offset_x
        y0 = y_max / 2 - height / 2
        x1 = x_max - offset_x
        y1 = y_max / 2 + height / 2
    elif gravitation == "SW":
        x0 = offset_x
        y0 = y_max - height - offset_y
        x1 = x0 + width
        y1 = y_max - offset_y
    elif gravitation == "S":
        x0 = x_max / 2 - width / 2
        y0 = y_max - height - offset_y
        x1 = x_max / 2 + width / 2
        y1 = y_max - offset_y
    elif gravitation == "SE":
        x0 = x_max - width - offset_x
        y0 = y_max - height - offset_y
        x1 = x_max - offset_x
        y1 = y_max - offset_y
    else:
        x0 = 5
        y0 = 5
        x1 = x_max - 5
        y1 = y_max - 5
    return (x0, y0, x1, y1)


def list_of_images(cwd, operating_system):
    """
    jpg, png and tiff images in cwd
    return sorted list
    """

    list_of_files = os.listdir(cwd)
    file_list = []
    patterns = ("*.JPG", "*.JPEG", "*.PNG", "*.TIF", "*.TIFF")
    if operating_system != "Windows":
        patterns = patterns + ("*.jpg", "*.jpeg", "*.png", "*.tif", "*.tiff")

    for pattern in patterns:
        for file in list_of_files:
            if fnmatch.fnmatch(file, pattern):
                file_list.append(file)

    file_list.sort()
    return file_list


def file_from_list_of_images(file_list, current_file, request):
    """
    return filename from file_list depends of request
    request: position on the list
    """
    if file_list:
        if request == "first":
            file = file_list[0]
        elif request == "previous":
            if current_file in file_list:
                position = file_list.index(current_file)
                if position > 0:
                    file = file_list[position - 1]
                else:
                    file = None
            else:
                file = file_list[0]
        elif request == "next":
            if current_file in file_list:
                position = file_list.index(current_file)
                if position <= len(file_list) - 2:
                    file = file_list[position + 1]
                else:
                    file = None
            else:
                file = file_list[-1]
        elif request == "last":
            file = file_list[-1]
        else:
            file = None
    else:
        file = None

    if file == current_file:
        file = None
    return file


def arrow_gravity(position, length, x0, y0):
    """calculate coordinated to draw arrow"""
    length = int(length)
    x0 = int(x0)
    y0 = int(y0)
    width = int(length / 3 / 2)
    length_1_2 = int(length / 2)
    length_1_3 = int(length / 3)
    length_1_4 = int(length / 4)

    offset_x = 0
    offset_y = 0
    c = (x0, y0)
    if position == "N":
        a = (x0, y0 + length)
        d = (x0 - width, y0 + length_1_3)
        e = (x0 + width, y0 + length_1_3)
        offset_y = length
    elif position == "S":
        a = (x0, y0 - length)
        d = (x0 - width, y0 - length_1_3)
        e = (x0 + width, y0 - length_1_3)
        offset_y = -length
    elif position == "W":
        a = (x0 + length, y0)
        d = (x0 + length_1_3, y0 - width)
        e = (x0 + length_1_3, y0 + width)
        offset_x = length
    elif position == "E":
        a = (x0 - length, y0)
        d = (x0 - length_1_3, y0 - width)
        e = (x0 - length_1_3, y0 + width)
        offset_x = -length
    elif position == "NW":
        a = (x0 + length, y0 + length)
        d = (x0 + length_1_4, y0 + length_1_2)
        e = (x0 + length_1_2, y0 + length_1_4)
        offset_x = length
        offset_y = length
    elif position == "NE":
        a = (x0 - length, y0 + length)
        d = (x0 - length_1_4, y0 + length_1_2)
        e = (x0 - length_1_2, y0 + length_1_4)
        offset_x = -length
        offset_y = length
    elif position == "SE":
        a = (x0 - length, y0 - length)
        d = (x0 - length_1_4, y0 - length_1_2)
        e = (x0 - length_1_2, y0 - length_1_4)
        offset_x = -length
        offset_y = -length
    elif position == "SW":
        a = (x0 + length, y0 - length)
        d = (x0 + length_1_4, y0 - length_1_2)
        e = (x0 + length_1_2, y0 - length_1_4)
        offset_x = length
        offset_y = -length
    else:
        a = (0, 0)
        d = (0, 0)
        e = (0, 0)

    msg = (position, a, c, d, e, offset_x, offset_y)
    module_logger.debug("arrow_gravity: %s", msg)
    return (a, c, d, e, offset_x, offset_y)


# EOF



# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

pkg_vars = {}
with open("fotokilof/version.py") as fp:
    exec(fp.read(), pkg_vars)
setup(
    name=pkg_vars['__appname__'],
    version=pkg_vars['__version__'],
    author=pkg_vars['__author__'],
    author_email=pkg_vars['__email__'],
    description=pkg_vars['__description__'],
    keywords=pkg_vars['__keywords__'],
    url=pkg_vars['__url__'],
    long_description=long_description,
    long_description_content_type="text/markdown",
    license="MIT",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.9',
    include_package_data=True,
    packages=find_packages(),
    install_requires=['FindSystemFontsFilename','pillow','requests','ttkbootstrap','wand'],
    entry_points = {
        "gui_scripts": [
            "fotokilof = fotokilof:__main__",
        ]
    },
)
